% -----------------------------code-info----------------------------------%
% This code processes zebrascope data obtained with the V3 software
%
% ---- outputs----%
% The code writes two multi-frame tiffs. 1)All raw frames 2)Selected frames 
% These tiffs can be further analyzed or processed in ImageJ
%
% The code writes an excel file with selected images. It has two columns
%           column 1: raw number of images that were selected
%           column 2: mean sarcomere length in the selected image
%----- notes -----%
% Created by Zebra Medical Technologies 
% Given to the ARMS lab by Gabriel Sanchez 
% Last modified by Amy Adkins:Changed the format of the output
% table (see % added by AA )
% ------------------------------------------------------------------------%

clear all;
close all;
clc;

%--------------------------------------------------------------------------
%load raw data file and make multi-page tiff of raw data

varianceprocess = true;

[FileName,PathName] = uigetfile('*.dat','Select the 2-photon data file')
tempfilename = findstr('.', FileName)
barefilename = FileName(1:tempfilename(end)-1)
rootfilename = [PathName barefilename];
textfilename = sprintf('%s.txt',rootfilename);
datafilename = sprintf('%s.dat',rootfilename);
[FrameWidth FrameHeight DwellTime InputRange ObjectiveMag ScanLens MaxVoltage AngleAtMax] = textread(textfilename,'%d %d %f %f %f %f %f %f',1,'delimiter','\t', ...
'headerlines',1,'emptyvalue',NaN);
fid0 = fopen(datafilename,'r');

width = FrameWidth;
length = FrameHeight;

endoffile = 0;
type='uint8';

prompt{1} = 'How many frames to average?';
dialoglabel = 'Process raw image data';
default_ans = {'1'};
answer = inputdlg(prompt,dialoglabel,1,default_ans);

nAve = str2num(answer{1});

if nAve == 1,
    aveString = '_noavg_';
else
    aveString = sprintf('_avg%d_',nAve);
end
count = 1;
FrameProcessed = 0;
raw_image = fread(fid0, width*length,type);
%raw_image = 255-raw_image;
endoffile = feof(fid0);  
while endoffile == 0,
    tempImage = reshape(raw_image, width, length);
    if varianceprocess == true,
        for count3 = 1:length,
            for count4 = 1:width,
                % pixelvalue(count,count3*width+count4) = tempImage(count3,count4);
            end
        end
    end
    if nAve == 1,

        if count == 1,
            imwrite(uint8(tempImage),[rootfilename aveString 'rawdata.tiff'],'tiff','Compression', 'none');
        else
            imwrite(uint8(tempImage),[rootfilename aveString 'rawdata.tiff'],'tiff','Compression', 'none','WriteMode','append'); 
        end
     
    else
        myImage(:,:,mod(count,nAve)+1) = tempImage;        
        if count >= nAve,
            aveImage = 0;
            for count2 = 1:nAve,
                aveImage = aveImage + myImage(:,:,count2);
            end
            aveImage = aveImage/nAve;
            if count == (nAve+1),  % first one
                imwrite(uint8(aveImage),[rootfilename aveString 'rawdata.tiff'],'tiff','Compression', 'none');
            else
                imwrite(uint8(aveImage),[rootfilename aveString 'rawdata.tiff'],'tiff','Compression', 'none','WriteMode','append');         
            end
        else
            myImage(:,:,count) = tempImage;
        end
    end
    count = count+1;
    FrameProcessed = FrameProcessed + 1
    raw_image = fread(fid0, width*length,type);
   % raw_image = 255-raw_image;
    endoffile = feof(fid0);  
end
fclose(fid0)

%--------------------------------------------------------------------------
%pick out the frames that have sarcomeres

prompt{1} = 'Shortest length (um)';
prompt{2} = 'Longest length (um)'; 
prompt{3} = 'Cutoff score: 0.2 - 0.5';
dialoglabel = 'Settings to identify sarcomere images';
default_ans = {'2','5','0.4'};
answer = inputdlg(prompt,dialoglabel,1,default_ans);

info = imfinfo([rootfilename aveString 'rawdata.tiff'],'TIFF');
num_images = numel(info);

shortsarc = str2num(answer{1});
longsarc = str2num(answer{2});
cutoff = str2num(answer{3});
sarcframes = 1;
FrameProcessed = 0;

%%

for k = 1:num_images;

%take the FFT of the kth frame
tempImage = imread([rootfilename aveString 'rawdata.tiff'], k);
imagesize = size(tempImage);
numpixels = imagesize(1);

%clear the edges and vertical center of noise and smooth
FT_of_Temp = abs(fftshift(fft2(tempImage)));

clearedges  = 50; %number of pixels from the image edge for HF region bounds
FT_of_Temp(1:clearedges,:) = 0; % clearing edges 
FT_of_Temp(:,1:clearedges) = 0; % clearing edges 
FT_of_Temp(numpixels - clearedges+1:numpixels,:) = 0; % clearing edges 
FT_of_Temp(:,numpixels - clearedges+1:numpixels) = 0; % clearing edges 
FT_of_Temp(:,(numpixels/2)) = 0; % clearing verticle center 

kernel = fspecial('gaussian', 10, 2); %smooth out raw FFT % rotationally symmetric low pass gaussian filter, kernel is 10 by 10, sigma of 2 pixles
FT_of_Temp_smooth = imfilter(FT_of_Temp, kernel); 
scalefactor = max(max(FT_of_Temp_smooth));
FT_of_Temp_smooth = (255/scalefactor)*FT_of_Temp_smooth; %scale temp image

%set frequency bounds for HF noise and sarc lengths
imagescale = 0.16;  %microns/pixel
highf = uint16(numpixels*imagescale/shortsarc);  %high freq- filter out shorter than 2.0 um  
lowf = uint16(numpixels*imagescale/longsarc);  %low freq- filter out longer than 6 um
HFlowerbound = 3*highf;

%calculate the white noise level
dcnoise1 = FT_of_Temp_smooth(clearedges+1:(numpixels/2)-HFlowerbound-1,clearedges+1:(numpixels/2)-HFlowerbound-1);
dcnoise2 = FT_of_Temp_smooth(clearedges+1:(numpixels/2)-HFlowerbound-1,(numpixels/2)+HFlowerbound+2:numpixels-clearedges);
dcnoise = [dcnoise1,dcnoise2];
whitenoise = mean(mean(dcnoise));
stdnoise = mean(std(dcnoise));

%threshold FFT at mean(whitenoise + 3*sd)
FT_of_Temp_smooth = FT_of_Temp_smooth - whitenoise; %subtract white noise mean
threshold = 3*stdnoise;
MassMatrix = zeros(numpixels,numpixels);
MassMatrix = MassMatrix + (stdnoise/10);

for i = 1:numpixels;
    for j = 1:numpixels;    
        if FT_of_Temp_smooth (i,j) > threshold;
            MassMatrix(i,j) = FT_of_Temp_smooth(i,j); %above threshold use greyscale value else 0 or black
        end
    end
end

%re-scale the mass matrix
MassMatrix = (255/max(max(MassMatrix)))*MassMatrix;

%calculate moment of inertia, normalized by mass
%only use pixels within the highf and lower range
%only use pixels within 45 degrees of vertical (i.e. hourgass shape)

mass = sum(sum(MassMatrix));
moment = 0;
acceptance_theta = 1;
scoredpixels = zeros(1,4);%x,y,Mass,Gaussian product
% frequencyAll = zeros(1,4);
% sarclengthAll = zeros(1,4);
pixelcount = 1;

%calculate moment within the feasible image region
for i = 1:numpixels/2;
    for j = 1:numpixels;
        x = j-(numpixels/2);
        y = i-(numpixels/2);
        rsquared = double((x^2) + (y^2));
        r = sqrt(rsquared); % diagonal distance from the center to the point being evalucated
        theta = abs(y/x); % theta is setting up so that moment is calculated at 45 deg 
        
        if theta>acceptance_theta; % accepted_theta means x = y therfore at 45 deg
            if (rsquared > lowf*lowf) && (rsquared < highf*highf);
                %compute moment
                moment = moment + ((MassMatrix(i,j))*rsquared); % this is still a little unclear
                
                %multiply with gaussian and store value
                tempgrid = MassMatrix(i-4:i+5,j-4:j+5); % ???
                gausfit = kernel.*tempgrid; % fitting to gaussian 
                                
                scoredpixels(pixelcount,:) = [x,y,MassMatrix(i,j),sum(sum(gausfit))];
                frequencyAll(pixelcount,:) = sqrt(((scoredpixels(pixelcount,1))^2) + ((scoredpixels(pixelcount,2))^2)); % this is just pythagorean therom to find diagonal distance from the center
                sarclengthAll(pixelcount,:)= numpixels*imagescale/frequencyAll(pixelcount,:);
                
                pixelcount = pixelcount +1; 
                
                
                
            end          
        end
    end
end



momentofinertia = (moment/mass);

%calculate x and y stdevs for pixel position, weighted by intensity
xf = (scoredpixels(:,1)).*(scoredpixels(:,3)); % x * mass matrix at that location
x2f = xf.*(scoredpixels(:,1)); % xf squared
Sxf = sum(xf);
Sx2f = sum(x2f);

yf = (scoredpixels(:,2)).*(scoredpixels(:,3)); % y * mass matrix at that location
y2f = yf.*(scoredpixels(:,2));
Syf = sum(yf);
Sy2f = sum(y2f);

Sintensity = sum(scoredpixels(:,3));
SSx = sqrt((Sx2f - (Sxf*Sxf/Sintensity))/(Sintensity-1));
SSy = sqrt((Sy2f - (Syf*Syf/Sintensity))/(Sintensity-1));

score(k) = momentofinertia/(SSx*SSy);

%find best fit sarc length
sarcfits = scoredpixels(:,4);
[bestfit,index] = max(sarcfits);
frequency = sqrt(((scoredpixels(index,1))^2) + ((scoredpixels(index,2))^2)); % this is just pythagorean therom to find diagonal distance from the center
sarclength = numpixels*imagescale/frequency;
     
    if score(k) > cutoff;
        
        %make multi-page tiff out of selected frame
        if sarcframes == 1,  
                imwrite(tempImage,[rootfilename aveString 'SelectedFrames.tiff'],'tiff','Compression', 'none');
        else
                imwrite(tempImage,[rootfilename aveString 'SelectedFrames.tiff'],'tiff','Compression', 'none','WriteMode','append');         
        end
        
        meanlength(sarcframes) = sarclength;
        goodframes(sarcframes) = sarcframes;
        goodframes_imagenum(sarcframes) = k; % added by AA 
        
        sarcframes = sarcframes +1;
        
    end
    

FrameProcessed = FrameProcessed + 1

end;

rawframes = [1:num_images];
threshold = ones(1,num_images);
threshold = threshold*cutoff;

figure(1);
plot(rawframes,score,rawframes,threshold);

figure(2);
plot(goodframes,meanlength);

% commented out by AA %
% table = zeros(num_images,4);
% table(:,1) = rawframes;
% table(:,2) = score;
% table((1:sarcframes-1),3) = goodframes;
% table((1:sarcframes-1),4) = meanlength;

table((1:sarcframes-1),1) = goodframes_imagenum; % added by AA 
table((1:sarcframes-1),2) = meanlength; % added by AA 


filename = [rootfilename aveString 'SarcomereMeasurements.xlsx'];
SUCCESS = xlswrite(filename,table,1) 

mean(meanlength)
std(meanlength)
size(meanlength)
